package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioFinanceiroReceitasDespesasBean;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioFinanceiroReceitasDespesas extends Relatorio {
	
	private List<RelatorioFinanceiroReceitasDespesasBean> lsDadosRelatorio;
	
	public RelatorioFinanceiroReceitasDespesas(Map<String, Object> parametros, Connection conexao) throws Exception {
		
		super(parametros, conexao);
		
		this.setCustomDataSource(new DataSource(this) {
			 @Override
			public void initDados() throws Exception {
				super.initDados();
				lsDadosRelatorio = executeQuery();
				setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
			}
		});
		
	}
	
	@SuppressWarnings("unchecked")
	private List<RelatorioFinanceiroReceitasDespesasBean> executeQuery() throws SQLException {
		
		Date fecInicio = (Date) parametros.get("fecInicio");
		Date fecFinal = (Date) parametros.get("fecFinal");
		Empresa empresa = (Empresa) parametros.get("empresa");
		List<PuntoVenta> lsPuntoVenta = (List<PuntoVenta>) parametros.get("lsPuntoVenta");
		Integer indTipo = (Integer) parametros.get("indTipo");
		
		String sql = getSql(indTipo, lsPuntoVenta);
		
		NamedParameterStatement stmt = new NamedParameterStatement(this.getConexao(), sql);
		stmt.setDate("fecInicio", new java.sql.Date(fecInicio.getTime()));
		stmt.setDate("fecFinal", new java.sql.Date(fecFinal.getTime()));
		stmt.setInt("empresaId", empresa.getEmpresaId());
		
		ResultSet resultSet = stmt.executeQuery();
		List<RelatorioFinanceiroReceitasDespesasBean> lsBean = new ArrayList<RelatorioFinanceiroReceitasDespesasBean>();
		while (resultSet.next()) {
			RelatorioFinanceiroReceitasDespesasBean bean = new RelatorioFinanceiroReceitasDespesasBean();
			bean.setDescTipoEvento(resultSet.getString("DESCTIPOEVENTO"));
			bean.setFecHorVta(resultSet.getDate("FECHORVTA"));
			bean.setTipoEventoExtraId(resultSet.getInt("TIPOEVENTOEXTRA_ID"));
			bean.setNombPuntoVenta(resultSet.getString("NOMBPUNTOVENTA"));
			bean.setPrecio(resultSet.getBigDecimal("PRECIO"));
			bean.setPuntoVentaId(resultSet.getInt("PUNTOVENTA_ID"));
			lsBean.add(bean);
		}
		
		return lsBean;
		
	}
	
	private String getSql(Integer indTipo, List<PuntoVenta> lsPuntoVenta) {
		
		String sql = "SELECT C.FECHORVTA, P.PUNTOVENTA_ID, P.NOMBPUNTOVENTA, TE.INDTIPO, TE.TIPOEVENTOEXTRA_ID, TE.DESCTIPOEVENTO, C.PRECIO "
				+ "FROM CAJA_DIVERSOS C "
				+ "JOIN EVENTO_EXTRA E ON E.EVENTOEXTRA_ID = C.EVENTOEXTRA_ID "
				+ "JOIN TIPO_EVENTO_EXTRA TE ON TE.TIPOEVENTOEXTRA_ID = E.TIPOEVENTOEXTRA_ID "
				+ "JOIN PUNTO_VENTA P ON P.PUNTOVENTA_ID = C.PUNTOVENTA_ID "
				+ "WHERE E.EMPRESA_ID = :empresaId "
				+ "AND C.FECHORVTA BETWEEN :fecInicio AND :fecFinal ";
		
		if (indTipo >= 0) {
			sql += "AND TE.INDTIPO = " + indTipo;
		}
		if (CollectionUtils.isNotEmpty(lsPuntoVenta)) {
			Integer[] lsPuntoVentaId = new Integer[lsPuntoVenta.size()];
			for (int i = 0; i < lsPuntoVenta.size(); i++) {
				lsPuntoVentaId[i] = lsPuntoVenta.get(i).getPuntoventaId();
			}
			sql += "AND C.PUNTOVENTA_ID IN (" + StringUtils.join(lsPuntoVentaId, ',') + ") ";
		}
		
		return sql;
				
	}

	@Override
	protected void processaParametros() throws Exception {}

}
