package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioFinanceiroReceitasDespesas;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPuntoVentaSimple;
import com.trg.search.Filter;

/**
 * 
 * @author Lucas
 *
 */
@Controller("relatorioFinanceiroReceitasDespesasController")
@Scope("prototype")
public class RelatorioFinanceiroReceitasDespesasController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	@Autowired
	private DataSource dataSourceRead;

	@Autowired
	private EmpresaService empresaService;

	private List<Empresa> lsEmpresa;
	private Datebox dataInicial;
	private Datebox dataFinal;
	private Combobox cmbIndTipo;
	private Combobox cmbEmpresa;
	
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private MyListbox puntoVentaList;
	private MyListbox puntoVentaSelectedList;
	private Textbox txtPalavraPesquisa;
	private Paging pagingPuntoVenta;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		lsEmpresa = empresaService.obtenerTodos();
		puntoVentaList.setItemRenderer(new RenderPuntoVentaSimple());
		puntoVentaSelectedList.setItemRenderer(new RenderPuntoVentaSimple());
	}
	
	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		
		Map<String, Object> parametros = new HashMap<String, Object>();
		parametros.put("fecInicio", dataInicial.getValue());
		parametros.put("fecInicioFormatted", sdf.format(dataInicial.getValue()));
		parametros.put("fecFinal", dataFinal.getValue());
		parametros.put("fecFinalFormatted", sdf.format(dataFinal.getValue()));
		Empresa empresa = (Empresa) cmbEmpresa.getSelectedItem().getValue();
		parametros.put("empresa", empresa);
		parametros.put("nombEmpresa", empresa.getNombempresa());
		parametros.put("lsPuntoVenta", Arrays.asList(puntoVentaSelectedList.getData()));
		parametros.put("indTipo", Integer.valueOf((String) cmbIndTipo.getSelectedItem().getValue()));
		
		RelatorioFinanceiroReceitasDespesas relatorio = new RelatorioFinanceiroReceitasDespesas(parametros, dataSourceRead.getConnection());
		
		Map<String, Object> args = new HashMap<String, Object>();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				   Labels.getLabel("indexController.mniRelatorioVendasComissao.label"), args, MODAL);
		
	}
	
	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelectedList.addItemNovo(puntoVentaSel);
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaSelectedList.getSelected();
		puntoVentaSelectedList.removeItem(puntoVentaSel);
	}
	
	public void onClick$btnLimpar(Event ev) {
		puntoVentaSelectedList.setData(new ArrayList<Object>());
	}
	
	public void onClick$btnPesquisa(Event ev) {
		
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class,
						pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterOr(Filter.like("nombpuntoventa", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")), Filter.like("numPuntoVenta", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")));
		puntoVentaBusqueda.addSortAsc("nombpuntoventa");
		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioReceitaDiariaAgenciaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
		
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

}
